"""Constants of services that can be discovered."""

BELKIN_WEMO = "belkin_wemo"
DLNA = "DLNA"
GOOGLE_CAST = "google_cast"
PHILIPS_HUE = "philips_hue"
PMS = 'plex_mediaserver'
LMS = 'logitech_mediaserver'
NETGEAR_ROUTER = "netgear_router"
SONOS = "sonos"
PANASONIC_VIERA = "panasonic_viera"
SABNZBD = 'sabnzbd'
KODI = 'kodi'
HOME_ASSISTANT = "home_assistant"
MYSTROM = 'mystrom'
HASS_IOS = "hass_ios"
BOSE_SOUNDTOUCH = 'bose_soundtouch'
SAMSUNG_TV = "samsung_tv"
FRONTIER_SILICON = "frontier_silicon"
APPLE_TV = "apple_tv"
HARMONY = "harmony"

ATTR_NAME = 'name'
ATTR_HOST = 'host'
ATTR_PORT = 'port'
ATTR_HOSTNAME = 'hostname'
ATTR_URLBASE = 'urlbase'
ATTR_DEVICE_TYPE = 'device_type'
ATTR_MODEL_NAME = 'model_name'
ATTR_MODEL_NUMBER = 'model_number'
ATTR_PROPERTIES = 'properties'
ATTR_SSDP_DESCRIPTION = 'ssdp_description'
ATTR_SERIAL = 'serial'
ATTR_MAC_ADDRESS = 'mac_address'
